@@ -1,31 +1,36 @@
 #ifdef DTPY
   #include "py_fread.h"
 #else
-  #define STRICT_R_HEADERS
+  #ifndef STRICT_R_HEADERS
+    #define STRICT_R_HEADERS
+  #endif
   #include <R.h>
+  #include <Rinternals.h>  // for SEXP in writeList() prototype
   #include "po.h"
   #define STOP     error
   #define DTPRINT  Rprintf
+  static char internal_error_buff[256] __attribute__((unused)); // todo: fix imports such that compiler warns correctly #6468
+  #define INTERNAL_STOP(...) do {snprintf(internal_error_buff, sizeof(internal_error_buff), __VA_ARGS__); error("%s %s: %s. %s", _("Internal error in"), __func__, internal_error_buff, _("Please report to the data.table issues tracker"));} while (0)
 #endif
 
-typedef void (*writer_fun_t)(const void *, int64_t, char **);
+typedef void writer_fun_t(const void *, int64_t, char **);
 
 // in the order of writer_fun_t in fwriteR.c
-void writeBool8();
-void writeBool32();
-void writeBool32AsString();
-void writeInt32();
-void writeInt64();
-void writeFloat64();
-void writeComplex();
-void writeITime();
-void writeDateInt32();
-void writeDateFloat64();
-void writePOSIXct();
-void writeNanotime();
-void writeString();
-void writeCategString();
-void writeList();
+writer_fun_t writeBool8;
+writer_fun_t writeBool32;
+writer_fun_t writeBool32AsString;
+writer_fun_t writeInt32;
+writer_fun_t writeInt64;
+writer_fun_t writeFloat64;
+writer_fun_t writeComplex;
+writer_fun_t writeITime;
+writer_fun_t writeDateInt32;
+writer_fun_t writeDateFloat64;
+writer_fun_t writePOSIXct;
+writer_fun_t writeNanotime;
+writer_fun_t writeString;
+writer_fun_t writeCategString;
+writer_fun_t writeList;
 
 void write_chars(const char *source, char **dest);
 
@@ -75,7 +80,7 @@ typedef struct fwriteMainArgs
   int64_t nrow;
   // a vector of pointers to all-same-length column vectors
   const void **columns;
-  writer_fun_t *funs;      // a vector of writer_fun_t function pointers
+  writer_fun_t **funs;    // a vector of writer_fun_t function pointers
 
   // length ncol vector containing which fun[] to use for each column
   // one byte to use 8 times less cache lines than a vector of function pointers would do
@@ -84,7 +89,8 @@ typedef struct fwriteMainArgs
 
   const void *colNames;   // NULL means no header, otherwise ncol strings
   bool doRowNames;        // optional, likely false
-  const void *rowNames;   // if doRowNames is true and rowNames is not NULL then they're used, otherwise row numbers are output.
+  const void *rowNames;   // if doRowNames is true and rowNames is NULL then row numbers are output
+  uint8_t rowNameFun;     // when rowNames is not NULL, which writer to use for them
   char sep;
   char sep2;
   char dec;
@@ -103,14 +109,15 @@ typedef struct fwriteMainArgs
                           //   iff scipen >= 3=8-5
   bool squashDateTime;
   bool append;
-  int buffMB;             // [1-1024] default 8MB
+  int buffMB;             // [1-1024] default 8MiB
   int nth;
   bool showProgress;
   bool is_gzip;
+  int gzip_level;
   bool bom;
   const char *yaml;
   bool verbose;
+  bool forceDecimal;      // force writing decimal points for numeric columns
 } fwriteMainArgs;
 
 void fwriteMain(fwriteMainArgs args);
-
