@@ -81,7 +81,7 @@ SEXP gforce(SEXP env, SEXP jsub, SEXP o, SEXP f, SEXP l, SEXP irowsArg) {
   mask = (1<<bitshift)-1;
   highSize = ((ngrp-1)>>bitshift) + 1;
 
-  grp = (int *)R_alloc(nrow, sizeof(int));   // TODO: use malloc and made this local as not needed globally when all functions here use gather
+  grp = (int *)R_alloc(nrow, sizeof(*grp));   // TODO: use malloc and made this local as not needed globally when all functions here use gather
                                              // maybe better to malloc to avoid R's heap. This grp isn't global, so it doesn't need to be R_alloc
   const int *restrict fp = INTEGER(f);
 
@@ -117,8 +117,8 @@ SEXP gforce(SEXP env, SEXP jsub, SEXP o, SEXP f, SEXP l, SEXP irowsArg) {
     int bitshift = MAX(nb-8, 0);  // TODO: experiment nb/2.  Here it doesn't have to be /2 currently.
     int highSize = ((nrow-1)>>bitshift) + 1;
     //Rprintf(_("When assigning grp[o] = g, highSize=%d  nb=%d  bitshift=%d  nBatch=%d\n"), highSize, nb, bitshift, nBatch);
-    int *counts = calloc(nBatch*highSize, sizeof(int));  // TODO: cache-line align and make highSize a multiple of 64
-    int *TMP   = malloc(nrow*2l*sizeof(int)); // must multiple the long int otherwise overflow may happen, #4295
+    int *counts = calloc(nBatch*highSize, sizeof(*counts));  // TODO: cache-line align and make highSize a multiple of 64
+    int *TMP   = malloc(sizeof(*TMP) * nrow*2l); // must multiple the long int otherwise overflow may happen, #4295
     if (!counts || !TMP ) {
       free(counts); free(TMP); // # nocov
       error(_("Failed to allocate counts or TMP when assigning g in gforce")); // # nocov
@@ -163,15 +163,15 @@ SEXP gforce(SEXP env, SEXP jsub, SEXP o, SEXP f, SEXP l, SEXP irowsArg) {
     //Rprintf(_("gforce assign TMP [ (o,g) pairs ] back to grp took %.3f\n"), wallclock()-started); started=wallclock();
   }
 
-  high = (uint16_t *)R_alloc(nrow, sizeof(uint16_t));  // maybe better to malloc to avoid R's heap, but safer to R_alloc since it's done via eval()
-  low  = (uint16_t *)R_alloc(nrow, sizeof(uint16_t));
+  high = (uint16_t *)R_alloc(nrow, sizeof(*high));  // maybe better to malloc to avoid R's heap, but safer to R_alloc since it's done via eval()
+  low  = (uint16_t *)R_alloc(nrow, sizeof(*low));
   // global ghigh and glow because the g* functions (inside jsub) share this common memory
 
   gx = (char *)R_alloc(nrow, sizeof(Rcomplex));  // enough for a copy of one column (or length(irows) if supplied)
   // TODO: reduce to the largest type present; won't be faster (untouched RAM won't be fetched) but it will increase the largest size that works.
 
-  counts = (int *)S_alloc(nBatch*highSize, sizeof(int));  // (S_ zeros) TODO: cache-line align and make highSize a multiple of 64
-  tmpcounts = (int *)R_alloc(getDTthreads(nBatch, false)*highSize, sizeof(int));
+  counts = (int *)S_alloc(nBatch*highSize, sizeof(*counts));  // (S_ zeros) TODO: cache-line align and make highSize a multiple of 64
+  tmpcounts = (int *)R_alloc(getDTthreads(nBatch, false)*highSize, sizeof(*tmpcounts));
 
   const int *restrict gp = grp;
   #pragma omp parallel for num_threads(getDTthreads(nBatch, false))   // schedule(dynamic,1)
@@ -192,7 +192,7 @@ SEXP gforce(SEXP env, SEXP jsub, SEXP o, SEXP f, SEXP l, SEXP irowsArg) {
     }
     uint16_t *restrict my_low = low + b*batchSize;
     int *restrict my_tmpcounts = tmpcounts + omp_get_thread_num()*highSize;
-    memcpy(my_tmpcounts, my_counts, highSize*sizeof(int));
+    memcpy(my_tmpcounts, my_counts, highSize*sizeof(*my_tmpcounts));
     for (int i=0; i<howMany; i++) {
       const int w = my_pg[i] >> bitshift;   // could use my_high but may as well use my_pg since we need my_pg anyway for the lower bits next too
       my_low[my_tmpcounts[w]++] = (uint16_t)(my_pg[i] & mask);
@@ -228,7 +228,7 @@ void *gather(SEXP x, bool *anyNA)
     #pragma omp parallel for num_threads(getDTthreads(nBatch, false))
     for (int b=0; b<nBatch; b++) {
       int *restrict my_tmpcounts = tmpcounts + omp_get_thread_num()*highSize;
-      memcpy(my_tmpcounts, counts + b*highSize, highSize*sizeof(int));   // original cumulated   // already cumulated for this batch
+      memcpy(my_tmpcounts, counts + b*highSize, highSize*sizeof(*my_tmpcounts));   // original cumulated   // already cumulated for this batch
       int *restrict my_gx = (int *)gx + b*batchSize;
       const uint16_t *my_high = high + b*batchSize;
       const int howMany = b==nBatch-1 ? lastBatchSize : batchSize;
@@ -257,7 +257,7 @@ void *gather(SEXP x, bool *anyNA)
       #pragma omp parallel for num_threads(getDTthreads(nBatch, false))
       for (int b=0; b<nBatch; b++) {
         int *restrict my_tmpcounts = tmpcounts + omp_get_thread_num()*highSize;
-        memcpy(my_tmpcounts, counts + b*highSize, highSize*sizeof(int));
+        memcpy(my_tmpcounts, counts + b*highSize, highSize*sizeof(*my_tmpcounts));
         double *restrict my_gx = (double *)gx + b*batchSize;
         const uint16_t *my_high = high + b*batchSize;
         const int howMany = b==nBatch-1 ? lastBatchSize : batchSize;
@@ -284,7 +284,7 @@ void *gather(SEXP x, bool *anyNA)
       #pragma omp parallel for num_threads(getDTthreads(nBatch, false))
       for (int b=0; b<nBatch; b++) {
         int *restrict my_tmpcounts = tmpcounts + omp_get_thread_num()*highSize;
-        memcpy(my_tmpcounts, counts + b*highSize, highSize*sizeof(int));
+        memcpy(my_tmpcounts, counts + b*highSize, highSize*sizeof(*my_tmpcounts));
         int64_t *restrict my_gx = (int64_t *)gx + b*batchSize;
         const uint16_t *my_high = high + b*batchSize;
         const int howMany = b==nBatch-1 ? lastBatchSize : batchSize;
@@ -313,7 +313,7 @@ void *gather(SEXP x, bool *anyNA)
     #pragma omp parallel for num_threads(getDTthreads(nBatch, false))
     for (int b=0; b<nBatch; b++) {
       int *restrict my_tmpcounts = tmpcounts + omp_get_thread_num()*highSize;
-      memcpy(my_tmpcounts, counts + b*highSize, highSize*sizeof(int));
+      memcpy(my_tmpcounts, counts + b*highSize, highSize*sizeof(*my_tmpcounts));
       Rcomplex *restrict my_gx = (Rcomplex *)gx + b*batchSize;
       const uint16_t *my_high = high + b*batchSize;
       const int howMany = b==nBatch-1 ? lastBatchSize : batchSize;
@@ -364,7 +364,7 @@ SEXP gsum(SEXP x, SEXP narmArg)
     const int *restrict gx = gather(x, &anyNA);
     ans = PROTECT(allocVector(INTSXP, ngrp));
     int *restrict ansp = INTEGER(ans);
-    memset(ansp, 0, ngrp*sizeof(int));
+    memset(ansp, 0, ngrp*sizeof(*ansp));
     bool overflow=false;
     //double started = wallclock();
     if (!anyNA) {
@@ -476,7 +476,7 @@ SEXP gsum(SEXP x, SEXP narmArg)
       const int64_t *restrict gx = gather(x, &anyNA);
       ans = PROTECT(allocVector(REALSXP, ngrp));
       int64_t *restrict ansp = (int64_t *)REAL(ans);
-      memset(ansp, 0, ngrp*sizeof(int64_t));
+      memset(ansp, 0, ngrp*sizeof(*ansp));
       if (!anyNA) {
         #pragma omp parallel for num_threads(getDTthreads(highSize, false))
         for (int h=0; h<highSize; h++) {
@@ -502,13 +502,13 @@ SEXP gsum(SEXP x, SEXP narmArg)
               const int64_t *my_gx = gx + b*batchSize + pos;
               const uint16_t *my_low = low + b*batchSize + pos;
               for (int i=0; i<howMany; i++) {
-                const int64_t elem = my_gx[i];
-                if (elem!=INT64_MIN) {
-                  _ans[my_low[i]] += elem;
-                } else {
-                  _ans[my_low[i]] = INT64_MIN;
-                  break;
+                if (_ans[my_low[i]] == INT64_MIN) continue;
+                const int64_t b = my_gx[i];
+                if (b == INT64_MIN) {
+                  if (!narm) _ans[my_low[i]] = INT64_MIN;
+                  continue;
                 }
+                _ans[my_low[i]] += b;
               }
             }
           }
@@ -625,9 +625,9 @@ SEXP gmean(SEXP x, SEXP narmArg)
       for (int i=0; i<ngrp; i++) ansp[i] /= grpsize[i];
     } else {
       // narm==true and anyNA==true
-      int *restrict nna_counts = calloc(ngrp, sizeof(int));
+      int *restrict nna_counts = calloc(ngrp, sizeof(*nna_counts));
       if (!nna_counts)
-        error(_("Unable to allocate %d * %zu bytes for non-NA counts in gmean na.rm=TRUE"), ngrp, sizeof(int)); // # nocov
+        error(_("Unable to allocate %d * %zu bytes for non-NA counts in gmean na.rm=TRUE"), ngrp, sizeof(*nna_counts)); // # nocov
       #pragma omp parallel for num_threads(getDTthreads(highSize, false))
       for (int h=0; h<highSize; h++) {
           double *restrict _ans = ansp + (h<<bitshift);
@@ -678,8 +678,8 @@ SEXP gmean(SEXP x, SEXP narmArg)
       }
     } else {
       // narm==true and anyNA==true
-      int *restrict nna_counts_r = calloc(ngrp, sizeof(int));
-      int *restrict nna_counts_i = calloc(ngrp, sizeof(int));
+      int *restrict nna_counts_r = calloc(ngrp, sizeof(*nna_counts_r));
+      int *restrict nna_counts_i = calloc(ngrp, sizeof(*nna_counts_i));
       if (!nna_counts_r || !nna_counts_i) {
         // # nocov start
         free(nna_counts_r); free(nna_counts_i);
@@ -839,7 +839,8 @@ static SEXP gminmax(SEXP x, SEXP narm, const bool min)
             ansd[thisgrp] = elem;
         }
       }
-    }}
+    }
+  }
     break;
   case CPLXSXP:
     error(_("Type 'complex' has no well-defined min/max"));
@@ -972,7 +973,7 @@ static SEXP gfirstlast(SEXP x, const bool first, const int w, const bool headw)
     } else {                                                                                       \
       /* w>1 && !first not supported because -i in R means everything-but-i and gnthvalue */       \
       /* currently takes n>0 only. However, we could still support n'th from the end, somehow */   \
-      internal_error(__func__, "unanticipated case first=%d w=%d headw=%d", first, w, headw);                \
+      internal_error(__func__, "unanticipated case first=%d w=%d headw=%d", first, w, headw);      \
     }                                                                                              \
   }
   switch(TYPEOF(x)) {
@@ -1124,7 +1125,7 @@ SEXP gprod(SEXP x, SEXP narmArg) {
   const int n = nosubset ? length(x) : irowslen;
   //clock_t start = clock();
   if (nrow != n) error(_("nrow [%d] != length(x) [%d] in %s"), nrow, n, "gprod");
-  long double *s = malloc(ngrp * sizeof(long double));
+  long double *s = malloc(sizeof(*s) * ngrp);
   if (!s)
     error(_("Unable to allocate %d * %zu bytes for gprod"), ngrp, sizeof(long double)); // # nocov
   for (int i=0; i<ngrp; ++i) s[i] = 1.0;
@@ -1282,4 +1283,3 @@ SEXP gshift(SEXP x, SEXP nArg, SEXP fillArg, SEXP typeArg) {
   // consistency with plain shift(): "strip" the list in the 1-input case, for convenience
   return isVectorAtomic(x) && length(ans) == 1 ? VECTOR_ELT(ans, 0) : ans;
 }
-
